/***************************************************************************
 *   Copyright (C) 2008, 2009 by Stephane List                             *
 *   slist@lilotux.net                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "qwebcam.h"

QWebCam::QWebCam(QWidget * parent):QLabel(parent),
	timer_(this),
	counter_(0)
{
	capture_ = cvCaptureFromCAM(0);
	if (!capture_) 
  {
    qDebug()<<"cannot get webcam...";
    setText("No webcam!");
    setAlignment(Qt::AlignCenter);
    resize(320, 240);
    return;
  }	
	cvGrabFrame(capture_);                    
	//recupere une image, pour voir les dimensions et redimensionner
	image_ = cvRetrieveFrame(capture_);
	qDebug() << "image size : " << image_->width << "x" << image_->height;
	resize(image_->width, image_->height);

	timer_.start(100);
	connect(&timer_,SIGNAL(timeout()), this,SLOT(captureLoop()));
}

QWebCam::~QWebCam()
{
	cvReleaseCapture( &capture_ );
	qDebug() << "nb frames :" << counter_;
}

QImage QWebCam::Ipl2QImage(const IplImage *newImage)
{
	QImage qtemp;
	if (newImage && cvGetSize(newImage).width > 0)
	{
		int x;
		int y;
		char* data = newImage->imageData;

		qtemp= QImage(newImage->width, newImage->height,QImage::Format_RGB32 );
		for( y = 0; y < newImage->height; y++, data +=newImage->widthStep )
			for( x = 0; x < newImage->width; x++)
			{
				uint *p = (uint*)qtemp.scanLine (y) + x;
				*p = qRgb(data[x * newImage->nChannels+2],
						data[x * newImage->nChannels+1],data[x * newImage->nChannels]);
			}
	}
	return qtemp;	
}

void QWebCam::captureLoop() // appeler par le timer, recupere l'image courante de la webcam et convertit
{
	if ( !capture_ )
	{
		return;
	}
	cvGrabFrame(capture_);
	image_ = cvRetrieveFrame(capture_);
	if (image_) 
	{
		counter_ ++;
		qImage_ = Ipl2QImage(image_);
		setPixmap(QPixmap::fromImage(qImage_.scaled(width(),height()),Qt::AutoColor));
	}
}

void QWebCam::mouseDoubleClickEvent (QMouseEvent * /* event */)
{
	qDebug() << "double click received in QWebCam";
	if (isWindow()) {
		if (isFullScreen())
			showNormal();
		else
			showFullScreen();
	}
	else {
		emit doubleclick();
	}
}
